<h2>Forgot your password?</h2>

<%= simple_form_for(resource, as: resource_name, url: password_path(resource_name), html: { method: :post }) do |f| %>
  <%= f.error_notification %>

  <div class="form-inputs">
    <%= f.input :email,
                required: true,
                autofocus: true,
                input_html: { autocomplete: "email" } %>
  </div>

  <div class="form-actions">
    <%= f.button :submit, "Send me reset password instructions" %>
  </div>
<% end %>

<%= render "devise/shared/links" %>
